==> extra-sleuth-tracing-configuration-0.0.5-sources.jar: cn/patterncat/tracing/config/mvc/ExtraTagMvcInterceptor.java <==
package cn.patterncat.tracing.config.mvc;

import brave.SpanCustomizer;
import cn.patterncat.tracing.ExtraTracingProperties;
import cn.patterncat.tracing.helper.HttpBodyTraceHelper;
import cn.patterncat.tracing.component.servlet.MultiReadableHttpServletRequest;
//import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.HandlerMapping;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.Objects;
import java.util.Optional;

/**
 * Created by cat on 2018-12-04.
 */
@Configuration
@EnableConfigurationProperties(ExtraTracingProperties.class)
public class ExtraTagMvcInterceptor implements HandlerInterceptor, WebMvcConfigurer {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExtraTagMvcInterceptor.class);

    private final ExtraTracingProperties properties;

    public ExtraTagMvcInterceptor(ExtraTracingProperties properties) {
        this.properties = properties;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        SpanCustomizer span = (SpanCustomizer) request.getAttribute(SpanCustomizer.class.getName());

        span.tag(properties.getServletRequestQueryTag(),Objects.toString(request.getQueryString(),""));

        if(properties.isExtraServletHeaderEnabled()){
            if(properties.getTraceTagHeaderMappings() != null && !properties.getTraceTagHeaderMappings().isEmpty()){
                properties.getTraceTagHeaderMappings()
                        .entrySet()
                        .stream()
                        .forEach(entry -> {
                            String tagName = entry.getKey();
                            String headerNames = entry.getValue();
                            if(!StringUtils.hasText(tagName) || !StringUtils.hasText(headerNames)){
                                LOGGER.error("detect invalid traceTagHeaderMappings: {}",entry);
                                return ;
                            }
                            Optional<String> result = Arrays.stream(headerNames.split(properties.getTagMultiHeaderSeparator()))
                                    //NOTE 这里可能返回null
                                    .map(name -> Objects.toString(request.getHeader(name),""))
                                    .filter(e -> StringUtils.isEmpty(e) == false)
                                    .findFirst();

                            span.tag(tagName,result.orElse(""));
                        });
            }
        }

        if(properties.isTraceServletRequestBody() && request instanceof MultiReadableHttpServletRequest && HttpBodyTraceHelper.isContentTypePrintable(properties.getPrintableMediaSubTypes(),request.getContentType())){
            MultiReadableHttpServletRequest wrapper = (MultiReadableHttpServletRequest)request;
            try {
                byte[] data = StreamUtils.copyToByteArray(wrapper.getInputStream());
//                byte[] data = IOUtils.toByteArray(wrapper.getInputStream());
                String requestBody = HttpBodyTraceHelper.getContentAsString(data, properties.getMaxPrintBodyBytes(), request.getCharacterEncoding());
                span.tag(properties.getServletRequestBodyTag(),requestBody);
                LOGGER.debug("request body:{}",requestBody);
            } catch (IOException e) {
                LOGGER.error(e.getMessage(),e);
            }

        }

        //spring mvc best matching pattern
        if(properties.isTraceSpringMvcUrlPattern()){
            String urlPattern = Objects.toString(request.getAttribute(HandlerMapping.BEST_MATCHING_PATTERN_ATTRIBUTE),"");
            span.tag(properties.getSpringMvcUrlPatternTag(),urlPattern);
        }

        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this);
    }
}

==> extra-sleuth-tracing-configuration-0.0.8-sources.jar: cn/patterncat/tracing/config/mvc/ExtraTagMvcInterceptor.java <==
package cn.patterncat.tracing.config.mvc;

import brave.SpanCustomizer;
import cn.patterncat.tracing.ExtraTracingProperties;
import cn.patterncat.tracing.helper.HttpBodyTraceHelper;
import cn.patterncat.tracing.component.servlet.MultiReadableHttpServletRequest;
//import org.apache.commons.io.IOUtils;
import cn.patterncat.tracing.helper.MaskHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpRequest;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.HandlerMapping;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Supplier;

/**
 * Created by cat on 2018-12-04.
 */
@Configuration
@EnableConfigurationProperties(ExtraTracingProperties.class)
public class ExtraTagMvcInterceptor implements HandlerInterceptor, WebMvcConfigurer {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExtraTagMvcInterceptor.class);

    private final ExtraTracingProperties properties;

    public ExtraTagMvcInterceptor(ExtraTracingProperties properties) {
        this.properties = properties;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        SpanCustomizer span = (SpanCustomizer) request.getAttribute(SpanCustomizer.class.getName());

        //NOTE 这里要替换掉敏感的查询字段
        String maskQueryString = maskIfNeeded(request,Objects.toString(request.getQueryString(),""));
        span.tag(properties.getServletRequestQueryTag(),maskQueryString);

        if(properties.isExtraServletHeaderEnabled()){
            if(properties.getTraceTagHeaderMappings() != null && !properties.getTraceTagHeaderMappings().isEmpty()){
                properties.getTraceTagHeaderMappings()
                        .entrySet()
                        .stream()
                        .forEach(entry -> {
                            String tagName = entry.getKey();
                            String headerNames = entry.getValue();
                            if(!StringUtils.hasText(tagName) || !StringUtils.hasText(headerNames)){
                                LOGGER.error("detect invalid traceTagHeaderMappings: {}",entry);
                                return ;
                            }
                            Optional<String> result = Arrays.stream(headerNames.split(properties.getTagMultiHeaderSeparator()))
                                    //NOTE 这里可能返回null
                                    .map(name -> Objects.toString(request.getHeader(name),""))
                                    .filter(e -> StringUtils.isEmpty(e) == false)
                                    .findFirst();

                            span.tag(tagName,result.orElse(""));
                        });
            }
        }

        if(properties.isTraceServletRequestBody() && request instanceof MultiReadableHttpServletRequest && HttpBodyTraceHelper.isContentTypePrintable(properties.getPrintableMediaSubTypes(),request.getContentType())){
            MultiReadableHttpServletRequest wrapper = (MultiReadableHttpServletRequest)request;
            try {
                byte[] data = StreamUtils.copyToByteArray(wrapper.getInputStream());
//                byte[] data = IOUtils.toByteArray(wrapper.getInputStream());
                String requestBody = HttpBodyTraceHelper.getContentAsString(data, properties.getMaxPrintBodyBytes(), request.getCharacterEncoding());
                //NOTE 这里要替换掉敏感的查询字段
                String maskedRequestBody = maskIfNeeded(request,requestBody);
                span.tag(properties.getServletRequestBodyTag(),maskedRequestBody);
                LOGGER.debug("request body:{}",requestBody);
            } catch (IOException e) {
                LOGGER.error(e.getMessage(),e);
            }

        }

        //spring mvc best matching pattern
        if(properties.isTraceSpringMvcUrlPattern()){
            String urlPattern = Objects.toString(request.getAttribute(HandlerMapping.BEST_MATCHING_PATTERN_ATTRIBUTE),"");
            span.tag(properties.getSpringMvcUrlPatternTag(),urlPattern);
        }

        return true;
    }

    protected String maskIfNeeded(HttpServletRequest request, String query){
        return properties.mask(query, new Supplier<String>() {
            @Override
            public String get() {
                return request.getRequestURI();
            }
        });
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this);
    }
}
